function [apower,pdrop] = JQLA_pi

lambda = struct('mode','tb','lambda',0);
setting = struct('mu',0,'M',0,'W',0,'dt',0,'lambda',0,'options',0);
optPara = struct('TargetPbar',0,'piv0',0,'index',0,'rho_p',0,'Pmax',0);


%%%%%%%%%%% Change system parameters here %%%%%%%%%%%
setting.mu = 5; %# of bits per slot 
setting.M = 50; % Buffer size, bits 
setting.W = 5000; % bandwidth, Hz
setting.dt = 0.01; % slot duration, sec
setting.lambda.mode = 'Rayleigh';
setting.lambda.lambda = 1; % 1 / average channel gain
optPara.TargetPbar = 0.046; %average power
optPara.rho_p = 1;  % initial power control type, 1 for TDWF, 0 for TCI
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




setting.options = optimset('Display','on','TolFun',1e-10,'TolCon',1e-10,'MaxFunEvals',8000,'MaxIter',1000,'OutputFcn',@outfun);
% 'average': each row of the TP matrix has the same vaule 1 / min(i+\mu+1,
% M+1), this TP matrix can not guarantee the average power constraint
% 'combine': use rho_p to generate power control, this one make sure that
% the average power is smaller than the constraint
TV0 = get_initial_TV0(setting, optPara, 'combine');
optPara.piv0 = get_q_distr(TV0, setting);
optPara.Pmax = 1000;

tic;
% optimal pi
piv = min_drop_prob_pi(optPara, setting);
toc;

TV = optimal_pc(piv,setting);
% packet drop probability
pdrop = packet_drop_rate( TV, setting );
% average power
apower = ave_power(TV,setting);
% queue length distribution
q_dist = get_q_distr(TV, setting);

[gpower,ppower] = get_power_control_g_q(TV, setting);
draw_power_g_q(gpower, ppower, setting.M);

disp(['Packet drop probability', num2str( pdrop) ]);
disp(['Average power',num2str( apower) ]);


figure;
semilogy( [0:setting.M], q_dist );
xlabel('Queue Length');
ylabel('Queue length distribution');
grid on;

    function stop = outfun(x, optimValues, state)
        stop = false;
        % Check if directional derivative is less than .01.
        if optimValues.constrviolation / optPara.TargetPbar > 0.01
            stop = true;
        end
    end

end