% Return the transition probability matrix P
%
% x=[x(1),x(2),x(3)], the parameters for the CGB power control law
% x(1): alpha1,  x(2): alpha2   x(3): Cutoff
%   P(g) = alpha1 + alpha2 / g    (g > Cutoff)
%        = 0                      (g < Cutoff)
%
% Para: the structure containing the system parameters

function P = get_TP_matrix(Para)

mu=Para.mu; %# of bits per slot 
M=Para.M; % Buffer size 

W=Para.W; % bandwidth
dt=Para.dt; % slot duration

lambda=Para.lambda; % channel parameter

H = Para.H;
Pbar = Para.Pbar;

alpha1 = Para.alpha1;
alpha2 = Para.alpha2;
Cutoff = Para.Cutoff;


% pr(i,j) = Prob[ s(n) == j-1 | q(n) = i-1]
pr = zeros(M+1,M+1);

for queue = 0:M
    a1 = H(queue+1) * alpha1;
    a2 = 1 + H(queue+1) * alpha2;
    
    if H(queue+1) ~= 0
        
        if a1 ~= 0
            i = [1:M];
            b = 1 / a1 *  ( 2.^(i/W/dt) - a2 );
            index = find(b<0);
            b(index) = 0;
            if Cutoff < b(1)
                pr(queue+1 , 1) = 1 - CHCCDF(b(1), lambda); %exp( - lambda * b(1) );
            else
                pr(queue+1, 1) = 1 - CHCCDF(Cutoff, lambda); %exp( - lambda * Cutoff ) ;
            end

            for loop = 2:M
                if Cutoff  < b(loop - 1)
                    pr(queue+1, loop) =  CHCCDF( b(loop-1), lambda)- CHCCDF( b(loop), lambda); %exp( - lambda * b(loop-1) ) - exp( - lambda * b(loop) );
                elseif [ Cutoff >= b(loop-1) && Cutoff < b(loop) ]
                    pr(queue+1, loop) =  CHCCDF( Cutoff, lambda)- CHCCDF( b(loop), lambda); %exp( - lambda * Cutoff ) - exp( - lambda * b(loop) );
                elseif Cutoff >= b(loop)
                    pr(queue+1, loop) = 0;
                end
            end
            pr(queue+1, M+1) = 1 - sum( pr(queue+1, 1:M) );        
        else  % if a1 == 0 TCI
            temp = floor( W * dt  * log2( 1 + alpha2 * H(queue+1) ) );
            if temp > M
                temp = M;
            end
            pr(queue+1,temp+1) = CHCCDF( Cutoff, lambda); %exp( -lambda * Cutoff);
            pr(queue+1,1) = 1 - CHCCDF( Cutoff, lambda);
        end
    else   % H(queue+1) = 0
        pr(queue+1,  1 ) = 1;
    end
end


P = zeros(M+1,M+1);
for i = 0:M
        if i<M-mu
            j = [1:i+mu];
            P( i+1 , j + 1 ) = pr(i+1, i + mu - j + 1 );
            P( i+1 , 0 + 1 ) = sum(pr(i+1,i+mu+1:M+1));
        else
            P(i+1,1) = pr(i+1,M+1);
            j = [1:M];
            P(i+1,j+1) = pr(i+1,M-j+1);
        end
end
