% ??????
%
% min Average Power
% {Pi,j}
%
% s.t.  pi = pi P
%
% ?? 0 <= P{i,j} <= 1???
function TV = optimal_pc_prac(ps_theory,A,setting)
% A start from q = 0

lambda = setting.lambda;
mu = setting.mu;
M = setting.M;
piv = reshape(ps_theory,1,M+1);
index = get_index_of_TP( M, mu );

q = 0:1:M;
L = min( q+mu, M);  % vector L starts from L(0)
U = max( q-mu, 0);  % vector U starts from U(0)
B = A(2:end) - A(1:end-1);    
B = [0,B];   %  vector B starts from B(0)

y_hat = zeros(1,M);   % vector y_hat starts from y_hat(1)
options = optimset('Display','off','TolFun',1e-20,'TolCon',1e-20,'MaxFunEvals',100000,'MaxIter',1000);
for t = 1:M
     %y_hat(t) = fsolve(@(x) solv_y_hat(x,piv,t,B,M,mu,lambda), 0, options);
     [y_hat(t), fval] = fzero(@(x) solv_y_hat(x,piv,t,B,M,mu,lambda), [0,1000000], options);
     if abs( fval ) > 1e-5
         error('y_hat error');
     end
end
y_hat = [ y_hat, 0];

TP = zeros(M+1,M+1);

for qi = 0:M
    for qj = 1:L( qi + 1 )
        TP( qi + 1, qj + 1) = CHCCDF(    B( L(qi+1) - qj + 1 ) * y_hat( qj + 1 ),  lambda) ...
                              - CHCCDF(  B( L(qi+1) - qj + 2 ) * y_hat( qj ), lambda);
                              %exp( -lambda * B( L(qi+1) - qj + 1 ) * y_hat( qj + 1 ) ) - exp( -lambda * B( L(qi+1) - qj + 2 ) * y_hat( qj  ) );  
    end
    TP( qi + 1, 0 + 1 ) = 1 - sum( TP( qi + 1, 2:end ) );
end

TP( TP > -1e-10 & TP < 0 ) = 0;
if all( TP(:) >= 0 )
    TV = get_vector_from_TP_matrix(TP,index);
    return;
end

y_star = y_hat; % vector y_star starts from y_star(1)
for qj = M-1 : -1 : 1
    if TP( U(qj+1) + 2 , qj+1) < 0
        temp = fliplr( TP( U(qj+1) + 2, 1 : qj+1 ) );
        temp = fliplr( cumsum( temp ) );
        index_qj = find( temp >= 0, 1, 'last' )  - 1;
        if isempty( index_qj)
            index_qj = 0;
        end
        
        TP( ( U(qj+1) + 1 : M ) + 1, index_qj+1 ) = sum(   TP( (  U(qj+1) + 1 : M ) + 1, ( index_qj : qj )+1 ), 2  );
        for k = qj : -1 : (index_qj +1)  % column
            TP( ( U(qj+1) + 1 : M ) + 1, k+1 ) = 0;
            %y_star(k) = fsolve(@(x) solv_y_star(x,piv,k,qj,B,M,mu,lambda), 1, options);
            [y_star(k), fval] = fzero(@(x) solv_y_star(x,piv,k,qj,B,M,mu,lambda), [0,10000], options);
            if abs( fval ) > 1e-5
                error('y_hat error');
            end
            for i = U(k+1) : U(qj+1)    % row
                TP( i + 1, k + 1) = CHCCDF( B( L(i+1) - k + 1 ) * y_star( k + 1 ),  lambda) ...
                                  - CHCCDF( B( L(i+1) - k + 2 ) * y_star( k ), lambda);
                %exp( -lambda * B( L(i+1) - k + 1 ) * y_star( k + 1 ) ) - exp( -lambda * B( L(i+1) - k + 2 ) * y_star( k ) );
            end
        end
        if index_qj  > 0
            for i = U(index_qj+1) : U(qj+1)    % row
                TP( i + 1, index_qj + 1) =  CHCCDF( B( L(i+1) - index_qj + 1 ) * y_star( index_qj + 1 ),  lambda) ...
                                          - CHCCDF( B( L(i+1) - index_qj + 2 ) * y_star( index_qj ), lambda);
                %exp( -lambda * B( L(i+1) - index_qj + 1 ) * y_star( index_qj + 1 ) ) - exp( -lambda * B( L(i+1) - index_qj + 2 ) * y_star( index_qj ) );
            end
        else
            TP( :, 1) = 1 - sum( TP( :, 2:end ),2 );
        end
    end
end

TV = get_vector_from_TP_matrix(TP,index);




function y = solv_y_hat(x,piv,k,B,M,mu,lambda)
%%% piv = [pi0,pi1,....,piM]

j = 0:M;
U = max( j-mu, 0);  % vector U starts from U(0)
L = min( j+mu, M);  % vector L starts from L(0)

index_i =  U(k+1): M ;
index_B = L( index_i+1 ) - k + 1;
index_i2 =  U(k+1): k-1 ;

%y = piv( index_i + 1 ) * exp(  - lambda * B( index_B + 1 ) * x ).' - sum( piv( index_i2 + 1 ) );
y = piv( index_i + 1 ) * CHCCDF(  B( index_B + 1 ) * x, lambda ).' - sum( piv( index_i2 + 1 ) );


function y = solv_y_star(x,piv,k,k0,B,M,mu,lambda)
%%% piv = [pi0,pi1,....,piM]
j = 0:M;
U = max( j-mu, 0);  % vector U starts from U(0)
L = min( j+mu, M);  % vector L starts from L(0)

index_i =  U(k+1): U(k0+1) ;
index_B = L( index_i+1 ) - k + 1;
index_i2 =  k : k0 ;

%y = piv( index_i + 1 ) * ( 1 - exp(  - lambda * B( index_B + 1 ) * x ) ).' - sum( piv( index_i2 + 1 ) );
y = piv( index_i + 1 ) * ( 1 - CHCCDF(  B( index_B + 1 ) * x, lambda ) ).' - sum( piv( index_i2 + 1 ) );
