% [drop_rate, ps] = packet_drop_rate(H,Para)
% Given the queue length aware (QLA) power control law H and other system parameters, return the 
% packet drop rate 'drop_rate' and the queue length distribution 'ps'
% H: The QLA power control law. A (M+1)*1 column vector, when the queue
% length is i (i=0,1,...,M), the QLA power control law gives the power
% factor H(i+1).
% Para: the structure containing the system parameters
% drop_rate: average packet drop rate. It is defined as the ratio of the
% number of packets been dropped over the total arrival packets
% ps: queue length distribution. A (M+1)*1 column vector. The probability 
% of queue length equal to i (i=0,1,...,M) is ps(i+1)


function [drop_rate, ps] = packet_drop_rate_pi(piv,Para)

M = Para.M;
mu = Para.mu;

ps = piv;
ps = ps(:);
d = zeros(1,M+1);
index = [ (M - mu +1) : M];
d(index+1) = index + mu - M;
drop_rate = d * ps /mu;