% Calculate the cutoff threshold for time-domain water filling power
% control scheme  E_{g}( 1/c - 1/g ) = M
%
% c = water_filling(M, lambda)
% c = water_filling(M, lambda, c_range)
% c = water_filling(M, lambda, c_range, precision)
%
% M: average power
% lambda: channel pdf parameter
%
% c_range: [ c_min, c_max] initial test value
%   default c_min = 10e-10,   c_max = 10
%
% precision contrl the termination condition
% precision: [ rel_prec_power, c_prec  ]
%   rel_prec_power: relative precision of the average power :  default = 10e-4
%   c_prec: precision of c : default = 10e-6
%
% If the desired C is out of the range specified by c_range, -1 is returned 

function c = water_filling(M, lambda, c_range, precision)



if nargin < 3  % use default value
    % default value
    c_min = 10^(-10);
    c_max = 10;
    rel_prec_power = 10e-4;
    c_prec = 10e-6;
elseif nargin == 3
    c_min = c_range(1);
    c_max = c_range(2);
    rel_prec_power = 10e-4;
    c_prec = 10e-6;
elseif nargin == 4
    c_min = c_range(1);
    c_max = c_range(2);
    rel_prec_power = precision(1);
    c_prec = precision(2);
end

power_min = 1/c_min * CHCCDF(c_min, lambda) - CHpdfoverg(c_min, lambda);
power_max = 1/c_max * CHCCDF(c_max, lambda) - CHpdfoverg(c_max, lambda);

c = ( c_min + c_max ) / 2;
power = 1/c * CHCCDF(c, lambda) - CHpdfoverg(c, lambda);

while abs(power - M)/M > rel_prec_power && ( c_max - c_min ) > c_prec
    if power == M
        break;
    elseif power > M
        c_min = c;
    elseif power < M
        c_max = c;
    end
    c = ( c_min + c_max ) / 2;
    power = 1/c * CHCCDF(c, lambda) - CHpdfoverg(c, lambda);  %1/c * exp(-lambda*c)-lambda*expint(c*lambda);
end

if abs( power - M ) / M > rel_prec_power
    disp('Water filling C calculation error!');
    c = -1;
end




